use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Cookie;

use Xyzzy::Header;
use CGI::Util;

use Clarity -self;

field name;
field value;
field parameters => {};

my %boolean = (secure => undef, discard => undef);

sub headerize() {
	my @headerized;

	while(@_) {
		my $key = lc(shift);
		my $val = shift;
		if(exists $boolean{$key}) {
			next unless $val;
			undef $val;
		}
		$key =~ s/([a-z]+)/ucfirst($1)/ego;
		push @headerized, $key, $val;
	}

	return wantarray ? @headerized : \@headerized;
}

sub new {
	my $name = shift;
	my $value = shift;

	$self = super(name => $name, value => $value);
	$self->setparameter(Version => 1, @_);
	return $self;
}

sub setparameters {
	my $parameters = $self->parameters;
	while(@_) {
		my $key = lc(shift);
		$parameters->{$key} = shift;
	}
}

*setparameter = *setparameters;

sub unsetparameters {
	delete @{$self->parameters}{@_};
}

*unsetparameter = *unsetparameters;

sub as_header {
	my %values;

	return new Xyzzy::Header(
		$self->name => CGI::Util::escape($self->value),
		headerize(%{$self->parameters}));
}

sub toString { return $self->as_header->toString };

use overload
	'""' => \&toString,
	fallback => 1;
